(* Data type representing predicates *)

module Map = PMap.Map
module Set = PSet.Set

open Num
open Term
	
type predicate =
    Equality of term * term
  | Inequality of term
  | And of predicate * predicate
  | Or of predicate * predicate
  | Not of predicate

let pTrue = Inequality (term_of_num (num_of_int 0))
let pFalse = Inequality (term_of_num (num_of_int (-1)))
let pEquality(x,y) = Equality(x,y)
let pLeq(x,y) = Inequality(sub_term y x)
let pAnd(x,y) = if x = pTrue then y else if y = pTrue then x else if x = pFalse || y = pFalse then pFalse else And(x,y)
let pOr(x,y) = if x = pFalse then y else if y = pFalse then x else if x = pTrue || y = pTrue then pTrue else And(x,y)
let pNot(x) = if x = pTrue then pFalse else if x = pFalse then pTrue else Not(x)

(* form the conjunction of a list *)

let rec conj_list = function
    [] -> pTrue
  | [x] -> x
  | x :: y -> pAnd(x,(conj_list y))

(* form an implication *)

let implies p q = pOr(pNot(p),q)

(* return the set of symbols occurring in a predicate *)

let rec support_predicate =
  function
      Equality (x,y) -> Set.union (support_term x) (support_term y)
    | Inequality(x) -> support_term x
    | And(p,q) -> Set.union (support_predicate p) (support_predicate q)
    | Or(p,q) -> Set.union (support_predicate p) (support_predicate q)
    | Not p -> support_predicate p


(* is the support of term t contained int he support of predicates set ps? *)

let term_in_pred_set t ps =
  Set.subset (support_term t)
    (Set.fold ~f:(fun p s -> Set.union (support_predicate p) s) ~init:Set.empty ps)
    
(* construct x=y, or true if x and y are identical *)

let eq x y = if x = y then pTrue else pEquality(x,y)
